from pydantic import BaseModel, Field


class RunExperimentBody(BaseModel):
    experiment_id: str = Field(..., description="Id")
    requirements_file: str = Field(..., description="STAGING|PRODUCTION")
    model_pkl_file: str = Field(..., description="STAGING|PRODUCTION")


class ExperimentBody(BaseModel):
    experiment_name: str = Field(..., description="Id")
    experiment_tags: dict = Field(None, description="Tags")
    requirements_file: str = Field(None, description="STAGING|PRODUCTION")
    model_pkl_file: str = Field(None, description="STAGING|PRODUCTION")


class PredictModelBody(BaseModel):
    run_id: str = Field(None, description="Run ID")
    modelname: str = Field(..., description="DEPENDENCY|PREDICT")
    stage: str = Field(..., description="STAGING|PRODUCTION")



import logging

import dhuolib as dhuolib

model_name = "model_decision_tree"
tag = "1.0.1"
model = dhuolib.load_model(model_name, tag)


def prediction(inputs):
    data_transformed = None  # transform(inputs)
    predictions = model.predict(data_transformed)
    return predictions


def main():

    # seria interessante essa transformação chamar esse arquivo e passar os inputs como parametro
    inputs = dhuolib.read_from_bucket("s3://bucket1/inputs_encoded.csv")
    predictions = prediction(inputs)

    filename_predictions = "predictions.csv"

    predictions_result = dhuolib.save_predictions(
        filename_predictions, inputs, predictions, model_name, tag
    )

    logging.info(predictions_result)


if __name__ == "__main__":
    main()



import json

import requests


class ServiceAPIML:
    def __init__(self, service_endpoint):

        if not isinstance(service_endpoint, str):
            raise ValueError("service_endpoint must be a string")

        self.service_endpoint = f"{service_endpoint}/api/experiment"
        self.headers = {"Content-Type": "application/json"}

    def create_experiment_by_conf_json(self, experiment_params):
        if experiment_params is None and isinstance(experiment_params, dict):
            raise ValueError("json_data must be a dict")

        if 'model_pkl_file' in experiment_params.keys():
            return requests.post(
                f"{self.service_endpoint}/save", data=experiment_params, headers={"Content-Type": "multipart/form-data"}
            )
            
        return requests.post(f"{self.service_endpoint}/save", data=json.dumps(experiment_params), headers=self.headers)

    def run_experiment(self, params):
        if params is None and isinstance(params, str):
            raise ValueError("json_data must be a dict")
        
        response = requests.post(
            f"{self.service_endpoint}/run",
            data=params
        )
        return response

    def create_model(self, model_params):
        if model_params is None and not isinstance(model_params, dict):
            raise ValueError("json_data must be a dict")
        response = requests.post(
            f"{self.service_endpoint}/model",
            data=json.dumps(model_params),
            headers=self.headers,
        )
        return response

    def predict_online(self, params):
        if params is None and not isinstance(params, dict):
            raise ValueError("json_data must be a dict")
        response = requests.post(
            f"{self.service_endpoint}/predict_online", data=params, headers={"Content-Type": "multipart/form-data"}
        )
        return response



from dhuolib.clients import DhuolibClient
import sys
import unittest
from unittest.mock import patch

sys.path.append("src")


class TestDhuolibUtils(unittest.TestCase):
    def setUp(self):
        self.end_point = "http://localhost:8000"
        self.dhuolib = DhuolibClient(service_endpoint=self.end_point)
        self.file_path = "tests/files/LogisticRegression_best.pickle"

    def test_deve_lancar_excecao_com_valores_run_params_incorretos(self):
        experiment_params = {
            "experiment_tags": {"version": "v1", "priority": "P1"},
        }
        response = self.dhuolib.create_experiment(experiment_params)
        self.assertEqual(list(response.keys()), ["error"])

    @patch("requests.post")
    def test_deve_criar_o_experimento_com_run_params_corretos(self, mock_post):
        client = DhuolibClient(service_endpoint=self.end_point)

        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {"experiment_id": "1"}

        experiment_params = {
            "experiment_name": "test_experiment",
            "experiment_tags": {"version": "v1", "priority": "P1"}
        }

        response = client.create_experiment(experiment_params)
        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_deve_executar_o_experimento_com_run_params_corretos(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "experiment_id": "experiment_id",
            "run_id": "run_id",
            "model_uri": "model_uri",
        }

        run_params = {
            "experiment_id": "2",
            "model_pkl_file": "tests/files/LogisticRegression_best.pickle",
            "requirements_file": "tests/files/requirements.txt",
        }
        response = self.dhuolib.run_experiment(run_params)

        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_deve_criar_modelo_com_run_params_corretos(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "current_stage": "Production",
            "last_updated_timestamp": 1713582060414,
            "model_version": "1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "status": "READY",
        }

        run_params = {
            "stage": "Production",
            "modelname": "nlp_framework",
            "modeltag": "v1",
            "run_id": "9434e517ed104958b6f5f47d33c79184",
            "requirements_file": "tests/files/requirements.txt",
            "model_uri": "model_uri"
        }

        response = self.dhuolib.create_model(run_params)

        self.assertEqual(response, mock_response.json.return_value)

    @patch("requests.post")
    def test_deve_fazer_o_predict_online_a_partir_de_um_dataset(self, mock_post):
        mock_response = mock_post.return_value
        mock_response.status_code = 201
        mock_response.json.return_value = {
            "model_name": "nlp_framework",
            "predictions": [0, 1, 0, 1, 0, 1, 0, 1, 0, 1],
        }

        run_params = {
            "stage": "Production",
            "data": "tests/files/data_predict.csv",
            "modelname": "nlp_framework",

        }

        response = self.dhuolib.predict_online(run_params)

        self.assertEqual(response, mock_response.json.return_value)



class Auth:
    def login(self, user, pwd):
        print("User logged")

        token = "123123zcsfsfsdf1"
        return token

    def is_logged(self, token):
        return True


if __name__ == "__main__":
    auth = Auth()
    auth.login("Diego", "123")



from pydantic import ValidationError
import json
from dhuolib.config import logger
from dhuolib.services import ServiceAPIML
from dhuolib.validations import ExperimentBody, RunExperimentBody, PredictModelBody
from werkzeug.datastructures import FileStorage


class DhuolibClient:
    def __init__(self, service_endpoint=None):
        if not service_endpoint:
            raise ValueError("service_endpoint is required")

        self.service = ServiceAPIML(service_endpoint)

    def create_experiment(self, experiment_params: dict) -> dict:
        params = {}
        response = None

        try:
            ExperimentBody.parse_obj(experiment_params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        
        if "requirements_file" in experiment_params.keys() and "model_pkl_file" in experiment_params.keys():
            params["experiment_tags"] = json.dumps(
                experiment_params["experiment_tags"]
            )
            try:
                with open(experiment_params["requirements_file"], "rb") as f1, open(
                    experiment_params["model_pkl_file"], "rb"
                ) as f2:
                    params = {
                        "requirements_file": FileStorage(
                            stream=f1, filename="requirements.txt", content_type="text/plain"
                        ),
                        "model_pkl_file": FileStorage(
                            stream=f2,
                            filename="model.pkl",
                            content_type="application/octet-stream",
                        ),
                    }
                    params = {**params, **experiment_params}
    
                    response = self.service.create_experiment_by_conf_json(params)

                    experiment = response.json()
                    logger.info(
                        f"Experiment Name: {params['experiment_name']}"
                        f"Experiment ID: {experiment['experiment_id']} created"
                    )
                    return experiment
            except FileNotFoundError as e:
                logger.error(f"Error: {e}")
                return {"error": str(e)}
        params = {**params, **experiment_params}
        response = self.service.create_experiment_by_conf_json(params)
        experiment = response.json()
        logger.info(
            f"Experiment Name: {params['experiment_name']}"
            f"Experiment ID: {experiment['experiment_id']} created"
        )
        return experiment

    def run_experiment(self, params) -> dict:
        run_params = {}
        try:
            RunExperimentBody.parse_obj(params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        
        try:
            with open(params["requirements_file"], "rb") as f1, open(
                params["model_pkl_file"], "rb"
            ) as f2:
                run_params = {
                    "requirements_file": FileStorage(
                        stream=f1, filename="requirements.txt", content_type="text/plain"
                    ),
                    "model_pkl_file": FileStorage(
                        stream=f2,
                        filename="model.pkl",
                        content_type="application/octet-stream",
                    ),
                }
                
                run_params = {**run_params, **params}
                if run_params["experiment_id"] is None:
                    experiment_id = self.create_experiment(run_params)
                    run_params["experiment_id"] = experiment_id

                response = self.service.run_experiment(run_params)
                logger.info(f"Experiment ID: {run_params['experiment_id']} running")
                return response.json()
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def create_model(self, model_params) -> dict:
        try:
            if model_params["stage"] is None:
                model_params["stage"] = "STAGING"

            response = self.service.create_model(model_params)
            logger.info(f"Model Name: {model_params['modelname']} created")
            return response.json()
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def predict_online(self, run_params) -> dict:
        params = {}
        try:
            PredictModelBody.parse_obj(run_params)
        except ValidationError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}
        try:
            with open(run_params["data"], "rb") as f1:
                params = {
                    "data": FileStorage(
                        stream=f1, filename="data.csv", content_type="csv"
                    )
                }
                params = {**params, **run_params}
                response = self.service.predict_online(params)
                logger.info(f"Model Name: {params['modelname']} predictions")
                return response.json()
        except FileNotFoundError as e:
            logger.error(f"Error: {e}")
            return {"error": str(e)}

    def load_model(self, model_name, tag) -> str:
        print("loading model")
        return "model_name.pkl"

    def predict(self):
        pass

    def read_from_bucket(self, bucket: str, filename: str):
        # OCI, GCP, AWS
        return f"folder/{filename}"

    # def read_from_bucket(self, filename: str):
    #     default_bucket = "s3://default"
    #     return self.read_from_bucket(default_bucket, filename)

    def execute_select(self, statement: str):
        # Acesso pela API ML Service
        # Processo assíncrono
        file = "s3://bucket/temp/file1.csv"
        return file

    def execute_dml(self, statement: str):
        # rows_count - linhas afetadas

        rows_count = 0
        return rows_count

    # Promover o model_name para "production"

    # - [ ] criar cluster no Dataflow
    # - [ ] informar o dado de schedule e criação de aplicação Dataflow
    # - [ ] inserção de lista de dependencias: libs + zip + script.py

    # @hydra.main(config_path="config", config_name="deploy")
    def deploy(self, params):

        # Enfileira o serviços

        return None

    def save_predictions(
        self, dataset_predictions, inputs, predictions, model_name, tag
    ):

        return ""


if __name__ == "__main__":
    dhuolib = DhuolibClient()

    experiment = "classificacao para recomendação"
    experiment_id = dhuolib.create_experiment(experiment)

    file = "pickle.file"
    model_name = "model_decision_tree"
    tag = "1.0.1"

    save_status = dhuolib.save_model(experiment_id, file, model_name, tag)
    print(save_status)

    file = "pickle.file"
    model_name = "model_logistic"
    tag = "1.0.2"

    save_status = dhuolib.save_model(experiment_id, file, model_name, tag)
    print(save_status)

    file = dhuolib.load_model("model_decision_tree", "1.0.1")

    data_input = {"x": 1}
    predictions = dhuolib.predict("model_decision_tree", "1.0.1", data_input)

    dhuolib.deploy()
